const chai = require('chai')
const Utils = require('../utils.js').Utils;
const {By, until} = require('selenium-webdriver');

let utils = new Utils(process.argv);

utils.mode.forEach(MODE => {
    utils.browsers.forEach(browser => {

        // Not able to open mobile site in firefox, skip this case
        if (MODE === 'mobile' && browser === 'firefox') {
            return;
        }
        
        describe(`Test Create Group in ${browser} Browser - ${MODE}`, function() {
            let driver;

            // before starting first test case, create driver session, 
            // then login as root and navigate to My Groups activity
            before(async function() {
                driver = utils.getBrowserDriver(browser, MODE);
                await utils.navigateToMyGroups(driver);
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_my_groups.png');
            });

            // if any test case fails, then close the driver session
            // because the test case depends on success of previous test case
            this.afterEach(function() {
                if (this.currentTest.state === 'failed') {
                    driver.close();
                }
                utils.displayScreenshots(this);
            });

            // after all test cases are done, close the driver session
            after(function() {
                driver.close();
            });

            it('locate create group button, if found perform click', async function() {
                let createGroupSpan = await driver.wait(until.elementLocated(By.css('span[aria-label="Create Group"]')), 1000);
                await createGroupSpan.click();
                await utils.takeScreenshot(driver, MODE, browser, 'group_create.png');
            });

            it('open and close help frame', async function() {
                let helpBtn = await driver.findElement(By.css('button[type=button]'));
                await helpBtn.click();
                let helpFrame = await driver.findElement(By.id('help-frame'));
                let isVisible = await helpFrame.isDisplayed();
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_help.png');
                if (isVisible) {
                    await helpBtn.click();
                    isVisible = await helpFrame.isDisplayed();
                    chai.expect(isVisible).to.be.false;
                }
            });

            it('click Register drop down, select Anyone option', async function() {
                let registerDropDown = await driver.findElement(By.id('register-type'));
                await registerDropDown.click();
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_register_type.png');
                let anyoneOption = await driver.findElement(By.css('#register-type > option:nth-child(4)'));
                await anyoneOption.click();
                let selectedOption = await registerDropDown.getAttribute('value');
                chai.expect(selectedOption).to.equal('4');
            });

            it('click Access drop down, select Member Can Start Threads option', async function() {
                let accessDropDown = await driver.findElement(By.id('member-access'));
                await accessDropDown.click();
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_member_access.png');
                let option = await driver.findElement(By.css('#member-access > option:nth-child(3)'));
                await option.click();
                let selectedOption = await accessDropDown.getAttribute('value');
                chai.expect(selectedOption).to.equal('4');
            });

            it('click Voting drop down, select +/- Voting option', async function() {
                let votingDropDown = await driver.findElement(By.id('vote-access'));
                await votingDropDown.click();
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_voting_access.png');
                let option = await driver.findElement(By.css('#vote-access > option:nth-child(3)'));
                await option.click();
                let selectedOption = await votingDropDown.getAttribute('value');
                chai.expect(selectedOption).to.equal('2');
            });

            it('click Post Lifetime drop down, select Never Expires option', async function() {
                let plDropDown = await driver.findElement(By.id('post-lifetime'));
                await plDropDown.click();
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_post_lifetime.png');
                let option = await driver.findElement(By.css('#post-lifetime > option:nth-child(1)'));
                await option.click();
                let selectedOption = await plDropDown.getAttribute('value');
                chai.expect(selectedOption).to.equal('-2');
            });

            it('click Encryption drop down, select Disable option', async function() {
                let encryptDropDown = await driver.findElement(By.id('encryption'));
                await encryptDropDown.click();
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_encryption.png');
                let option = await driver.findElement(By.css('#encryption > option:nth-child(2)'));
                await option.click();
                let selectedOption = await encryptDropDown.getAttribute('value');
                chai.expect(selectedOption).to.equal('0');
            });

            it(`enter name - Test ${browser} - ${MODE}, click save`, async function() {
                let name = await driver.findElement(By.id('group-name'));
                await name.sendKeys(`Test ${browser} - ${MODE}`);
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_details.png');
                let save = await driver.findElement(By.id('focus-button'));
                await save.click();
            });

            it('confirm group is created', async function() {
                await utils.takeScreenshot(driver, MODE, browser, 'group_create_result.png');
                await driver.wait(until.elementLocated(By.className('group-title-item')), 1000);
                let groups = await driver.findElements(By.className('group-title-item'));
                let found = false;
                const loop = async () => {
                    utils.asyncForEach(groups, async (group) => {
                        let name = await group.getText();
                        if(name === `Test ${browser}`) {
                            found  = true;
                        }
                    });
                    chai.expect(found).to.be.true;
                };
                loop();
            });

        });
    });
});