const chai = require('chai')
const Utils = require('../utils.js').Utils;
const {By, until} = require('selenium-webdriver');

let utils = new Utils(process.argv);

utils.mode.forEach(MODE => {
    utils.browsers.forEach(browser => {

        // Not able to open mobile site in firefox, skip this case
        if (MODE === 'mobile' && browser === 'firefox') {
            return;
        }

        describe(`Test Join and Leave Group in ${browser} Browser - ${MODE}`, function() {
            let driver;

            // before starting first test case, create 
            // driver session, then login as root
            before(async function() {
                driver = utils.getBrowserDriver(browser, MODE);
                await utils.signin(driver);
            });

            // if any test case fails, then close the driver session
            // because the test case depends on success of previous test case
            this.afterEach(function() {
                if (this.currentTest.state === 'failed') {
                    driver.close();
                }
                utils.displayScreenshots(this);
            });

            // after all test cases are done, close the driver session
            after(function() {
                driver.close();
            });

            it('locate Join Groups button, if found perform click', async function() {
                let joinGroupsBtn = await driver.wait(until.elementLocated(By.css('span[aria-label="Join Groups"]')), 1000);
                let isDisplayed = await joinGroupsBtn.isDisplayed();
                chai.expect(isDisplayed).to.be.true;
                await joinGroupsBtn.click();
            });

            it('check groups available to join', async function() {
                // if no groups are available to join,
                // the test cases will fail. Run ManyUserExperiments.php 
                // in yioop code to setup multiple dummy groups
                await driver.wait(until.elementLocated(By.className('access-result')), 1000);
                await utils.takeScreenshot(driver, MODE, browser, 'group_join_before.png');
            });

            it('open information for first group, then close it', async function() {
                let infoBtn = await driver.findElement(By.css('span[aria-label=Information]'));
                await infoBtn.click();
                let componentTitle = await driver.wait(until.elementLocated(By.css('div[class=current-activity] > h2')), 1000);
                await utils.takeScreenshot(driver, MODE, browser, 'group_join_information.png');
                chai.expect(await componentTitle.getText()).to.equal('Group Information');
                let closeBtn = await driver.findElements(By.css('span[aria-label=Close]'));
                await closeBtn[closeBtn.length-1].click();
            });

            it('locate join button, if found perform click', async function() {
                let joinBtn = await driver.wait(until.elementLocated(By.css('span[aria-label="Join Group"]')),1000);
                chai.expect(await joinBtn.isDisplayed()).to.be.true;
                await joinBtn.click();
            });

            it('check message, confirm group joined', async function() {
                let messageDiv = await driver.wait(until.elementLocated(By.id('message')), 1000);
                chai.expect(await messageDiv.getText()).to.equal('Group Joined!');
            });

            it('navigate to my groups, locate leave group button, if found perform click', async function() {
                // in mobile mode the Groups button element is shadowed by the message div
                // displayed when a group is joined. Due to this Groups button becomes
                // uninteractable. So instead of directly navigating to My Groups activity,
                // close the current session, create new and then navigate to My Groups
                if(MODE === 'mobile') {
                    driver.close();
                    driver = utils.getBrowserDriver(browser, MODE);
                    await utils.navigateToMyGroups(driver);
                } else {
                    let groups = await driver.wait(until.elementLocated(By.css('span[aria-label="Groups"]')), 1000);
                    await groups.click();
                }
                await utils.takeScreenshot(driver, MODE, browser, 'group_join_after.png');
                let leaveGroupBtn = await driver.findElement(By.css('span[aria-label="Leave Group"]'));
                chai.expect(await leaveGroupBtn.getAttribute('aria-label')).to.equal('Leave Group');
                await leaveGroupBtn.click();
            });

            it(`create group page displayed, close it and confirm group is unsubscribed`, async function() {
                await utils.takeScreenshot(driver, MODE, browser, 'group_join_leave_message.png');
                let messageDiv = await driver.wait(until.elementLocated(By.id('message')), 1000);
                chai.expect(await messageDiv.getText()).to.equal('Successfully Unsubscribed!');
            });

        });
    });
});