const chai = require('chai')
const Utils = require('../utils.js').Utils;
const {By, until} = require('selenium-webdriver');

let utils = new Utils(process.argv);

utils.mode.forEach(MODE => {
    utils.browsers.forEach(browser => {

        // Not able to open mobile site in firefox, skip this case
        if (MODE === 'mobile' && browser === 'firefox') {
            return;
        }

        describe(`Test Threads for Group in ${browser} Browser - ${MODE}`, function() {
            let driver;
            
            // before starting first test case, create 
            // driver session, then login as root
            before(async function() {
                driver = utils.getBrowserDriver(browser, MODE);
                await utils.signin(driver);
            });

            // if any test case fails, then close the driver session
            // because the test case depends on success of previous test case
            this.afterEach(function() {
                if (this.currentTest.state === 'failed') {
                    driver.close();
                }
                utils.displayScreenshots(this);
            });

            // after all test cases are done, close the driver session
            after(function() {
                driver.close();
            });

            it('locate Group Discussions button for first group, if found perform click', async function() {
                let groupDiscussionsBtn = await driver.wait(until.elementLocated(By.css('span[aria-label="Group Discussions"]')), 1000);
                let isDisplayed = await groupDiscussionsBtn.isDisplayed();
                chai.expect(isDisplayed).to.be.true;
                await groupDiscussionsBtn.click();
            });

            it('locate start new thread button, if found perform click', async function() {
                await utils.takeScreenshot(driver, MODE, browser, 'thread.png');
                let startThreadBtn = await driver.wait(until.elementLocated(By.css('button[class=button-box]')), 1000);
                await startThreadBtn.click();
            });

            it('confirm thread details form is displayed', async function() {
                await utils.takeScreenshot(driver, MODE, browser, 'thread_start_new.png');
                await driver.wait(until.elementLocated(By.id('start-thread')), 1000);
            });

            it('enter subject and post details, click save', async function() {
                let subject = await driver.findElement(By.id('title-start-thread'));
                chai.expect(await subject.isDisplayed()).to.be.true;
                await subject.sendKeys(`Test ${browser} - ${MODE}`);
                let post = await driver.findElement(By.id('description-start-thread'));
                chai.expect(await post.isDisplayed()).to.be.true;
                await post.sendKeys(`Starting new thread in ${browser}`);
                let saveBtn = await driver.findElement(By.css('button[type=submit]'));
                await saveBtn.click();
            });

            it('confirm thread created, then click thread link', async function() {
                // on clicking save button for starting new thread
                // takes around 6-7 secs to create the thread, so
                // the wait time used here is 10 secs 
                let threadLink = await driver.wait(until.elementLocated(By.linkText(`Test ${browser} - ${MODE}`)), 10000);
                await utils.takeScreenshot(driver, MODE, browser, 'thread_add.png');
                await threadLink.click();
            });

            it('locate comment button, if found perform click', async function() {
                await utils.takeScreenshot(driver, MODE, browser, 'thread_comment_page.png');
                let commentBtn = await driver.wait(until.elementLocated(By.css('button[class=button-box]')), 1000);
                await commentBtn.click();
            });

            it('confirm add comment available, then enter comment and click save', async function() {
                await driver.wait(until.elementLocated(By.id('add-comment')), 1000);
                let comment = await driver.findElement(By.id('comment-add-comment'));
                await comment.sendKeys(`This is test comment in ${browser} and ${MODE}`);
                let saveBtn = await driver.findElement(By.css('button[type=submit]'));
                await saveBtn.click();
                await utils.takeScreenshot(driver, MODE, browser, 'thread_comment_add.png');
            });

            it('confirm comment added', async function() {
                let comments = await driver.findElements(By.css('div[id^=description]'));
                let latestComment = await comments[comments.length-1].getText();
                chai.expect(latestComment).to.equal(`This is test comment in ${browser} and ${MODE}`);
            });

            it('edit comment, confirm it is edited', async function() {
                let editElements = await driver.findElements(By.css('span[aria-label=Edit]'));
                let latestEdit = editElements[editElements.length-1];
                await latestEdit.click();
                let comment = await driver.wait(until.elementLocated(By.css('textarea[id^=description]')), 1000);
                await comment.clear();
                await comment.sendKeys(`This is edited comment in ${browser} and ${MODE}`);
                let saveBtn = await driver.findElement(By.css('button[type=submit]'));
                await saveBtn.click();
                await utils.takeScreenshot(driver, MODE, browser, 'thread_comment_edit.png');
                let comments = await driver.findElements(By.css('div[id^=description]'));
                let latestComment = await comments[comments.length-1].getText();
                chai.expect(latestComment).to.contain(`This is edited comment in ${browser} and ${MODE}`);
            });

            it('delete comment', async function() {
                let deleteElements = await driver.findElements(By.css('span[aria-label=Delete]'));
                let lastDelete = deleteElements[deleteElements.length-1];
                await lastDelete.click();
                await driver.wait(until.alertIsPresent(), 1000);
                let alertWindow = driver.switchTo().alert();
                await alertWindow.accept();
            });

            it('delete thread', async function() {
                await utils.takeScreenshot(driver, MODE, browser, 'thread_comment_delete.png');
                let threadDelete = await driver.findElement(By.css('span[aria-label=Delete]'));
                await threadDelete.click();
                let alertWindow = driver.switchTo().alert();
                await alertWindow.accept();
                await utils.takeScreenshot(driver, MODE, browser, 'thread_delete.png');
            });

        });
    });
});