/**
 * This file contains the method which are
 * frequently used by the test cases file
 */

const {Builder, By, until} = require('selenium-webdriver');
const fs = require('fs');
const path = require('path');
const firefox = require('selenium-webdriver/firefox');
const chrome = require('selenium-webdriver/chrome');
const { WebDriver } = require('selenium-webdriver');
const addContext = require('mochawesome/addContext');

class Utils {

    constructor(args) {
        this.FIREFOX_MOBILE_USER_AGENT = 'Mozilla/5.0 (iPhone; CPU iPhone OS 11_0_1 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A402 Safari/604.1';
        this.CHROME_MOBILE_USER_AGENT = 'user-agent=Mozilla/5.0 (iPhone; CPU iPhone OS 12_1 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) CriOS/56.0.2924.75 Mobile/14E5239e Safari/602.1';
        this.mode = ['web','mobile'];
        this.browsers = [];
        let arg_length = args.length;
        if (args[arg_length-3].split(':')[1] == 1) {
            this.browsers.push('firefox');
        }
        if (args[arg_length-2].split(':')[1] == 1) {
            this.browsers.push('chrome');
        }
        this.HOME_PAGE_LINK = 'http://localhost:' + args[arg_length-1].split(':')[1] + '/';
        this.SIGNIN_PAGE_LINK = this.HOME_PAGE_LINK + 'admin/';
        this.screenshots = new Map();
    }

    /**
     * Initialize a driver session for a given browser.
     * It starts the session in headless mode with a maximized
     * screen and either web or mobile view of the site
     * 
     * @param {string} browser A string representing the name of browser
     * @param {string} mode  A string respresenting either web or mobile view of the site
     * @returns An initialized driver session in headless mode
     */
    getBrowserDriver(browser, mode='web') {
        let builder = new Builder().forBrowser(browser);
        let driver;
        if (browser === 'firefox') {
            //even though the window size is set for firefox, it creates window of default size
            builder.setFirefoxOptions(new firefox.Options().windowSize({width:2800,height:1800}).headless());
        } else {
            if (mode === 'web') {
                builder.setChromeOptions(new chrome.Options().windowSize({width:1920,height:1080}).headless());
            } else {
                builder.setChromeOptions(new chrome.Options().windowSize({width:414,height:896}).headless().addArguments(this.CHROME_MOBILE_USER_AGENT));
            }
        }
        driver = builder.build();

        if (mode === 'web') {
            driver.manage().window().maximize();
        }
        return driver;
    }

    /**
     * Captures the screenshot and saves the result at corresponding location
     * 
     * @param {WebDriver} driver An object of webdriver session
     * @param {string} mode A string respresenting site either web or mobile
     * @param {string} browser A string respresenting name of the browser
     * @param {string} name A string respresenting the name of the screenshot
     */
    async takeScreenshot(driver, mode = 'web', browser, name) {
        let path = './results/' + mode + '/' + browser + '/' + name;
        await driver.takeScreenshot().then(function(data) {
            fs.writeFileSync(path, data, 'base64');
        });
        this.screenshots.set(name, path);
    }

    /**
     * Opens the My Group activity
     * 
     * @param {WebDriver} driver An object of webdriver session
     */
    async navigateToMyGroups(driver) {
        this.signin(driver);
        let groupsBtn = await driver.wait(until.elementLocated(By.css('span[aria-label=Groups]')), 5000);
        await groupsBtn.click();
    }

    /**
     * Performs signin using given username and password
     * 
     * @param {WebDriver} driver An object of webdriver session
     * @param {string} username Username
     * @param {string} password Password
     */
    async signin(driver, username='root', password='') {
        await driver.get(this.SIGNIN_PAGE_LINK);
        let signinForm = await driver.findElement(By.xpath('//*[@id="body-container"]/div[2]/form'));
        await driver.wait(until.elementIsVisible(signinForm), 1000);
        await driver.findElement(By.id('username')).sendKeys(username);
        await driver.findElement(By.id('password')).sendKeys(password);
        await driver.findElement(By.css('button')).click();
    }

    /**
     * Provides async version of forEach loop
     * 
     * @param {Array} array Array to loop on
     * @param {*} callback Callback function to perform on each array item
     */
    asyncForEach(array, callback) {
        for(let index=0; index < array.length; index++) {
            callback(array[index], index, array);
        }
    }

    displayScreenshots(context) {
        for(let [title, value] of this.screenshots) {
            addContext(context, {title:title, value:path.resolve(value)});
        }
        this.screenshots.clear();
    }
}

exports.Utils = Utils;